/*
 * Author: Hiroyuki Chishiro
 * License: 2-Clause BSD
 */
#ifndef __MINI_UART_H__
#define __MINI_UART_H__

#define PERIPHERAL_BASE 0xfe000000

#ifndef __ASSEMBLY__

struct gpio_pin_data {
  volatile uint32_t reserved; /* not used. */
  volatile uint32_t data[2];
};

struct gpio_regs {
  volatile uint32_t func_select[6];
  struct gpio_pin_data output_set;
  struct gpio_pin_data output_clear;
  struct gpio_pin_data level;
  struct gpio_pin_data ev_detect_status;
  struct gpio_pin_data re_detect_enable;
  struct gpio_pin_data fe_detect_enable;
  struct gpio_pin_data hi_detect_enable;
  struct gpio_pin_data lo_detect_enable;
  struct gpio_pin_data async_re_detect;
  struct gpio_pin_data async_fe_detect;
  volatile uint32_t reserved; /* not used. */
  volatile uint32_t pupd_enable;
  volatile uint32_t pupd_enable_clocks[2];
};

#define REGS_GPIO ((struct gpio_regs *)(PERIPHERAL_BASE + 0x00200000))

typedef enum gpio_func {
  GPIO_FUNC_INPUT = 0,
  GPIO_FUNC_OUTPUT = 1,
  GPIO_FUNC_ALT0 = 4,
  GPIO_FUNC_ALT1 = 5,
  GPIO_FUNC_ALT2 = 6,
  GPIO_FUNC_ALT3 = 7,
  GPIO_FUNC_ALT4 = 3,
  GPIO_FUNC_ALT5 = 2
} gpio_func;


#define GPIO_TXD 14
#define GPIO_RXD 15


struct aux_regs {
  volatile uint32_t irq_status;
  volatile uint32_t enables;
  volatile uint32_t reserved[14]; /* not used. */
  volatile uint32_t mu_io;
  volatile uint32_t mu_ier;
  volatile uint32_t mu_iir;
  volatile uint32_t mu_lcr;
  volatile uint32_t mu_mcr;
  volatile uint32_t mu_lsr;
  volatile uint32_t mu_msr;
  volatile uint32_t mu_scratch;
  volatile uint32_t mu_control;
  volatile uint32_t mu_status;
  volatile uint32_t mu_baud_rate;
};

#define REGS_AUX ((struct aux_regs *)(PERIPHERAL_BASE + 0x00215000))


#define BAUDRATE 115200
#define SYSTEM_CLOCK_FREQ (500 * 1000 * 1000)

void gpio_pin_set_func(uint8_t pin_number, gpio_func func);
void gpio_pin_enable(uint8_t pin_number);

void init_uart(void);
void uart_pol_putc(char c);
void myputs(char *str);


#endif /* !__ASSEMBLY__ */

#endif /* __MINI_UART_H__ */
