/*
 * Author: Hiroyuki Chishiro
 * License: 2-Clause BSD
 */
#include "raspberry-pi-4.h"

void gpio_pin_set_func(uint8_t pin_number, gpio_func func)
{
  uint8_t bit_start = (pin_number * 3) % 30;
  uint8_t reg = pin_number / 10;

  uint32_t selector = REGS_GPIO->func_select[reg];
  selector &= ~(0x7 << bit_start);
  selector |= (func << bit_start);

  REGS_GPIO->func_select[reg] = selector;
}

void delay(int n)
{
  while (n > 0) {
    n--;
  }
}

void gpio_pin_enable(uint8_t pin_number)
{
  REGS_GPIO->pupd_enable = 0;
  delay(150);
  REGS_GPIO->pupd_enable_clocks[pin_number >> 5] = 1 << (pin_number & 0x1f);
  delay(150);
  REGS_GPIO->pupd_enable = 0;
  REGS_GPIO->pupd_enable_clocks[pin_number >> 5] = 0;
}

void init_uart(void)
{
  gpio_pin_set_func(GPIO_TXD, GPIO_FUNC_ALT5);
  gpio_pin_set_func(GPIO_RXD, GPIO_FUNC_ALT5);

  gpio_pin_enable(GPIO_TXD);
  gpio_pin_enable(GPIO_RXD);

  REGS_AUX->enables = 1;
  REGS_AUX->mu_control = 0;
  REGS_AUX->mu_ier = 0;
  REGS_AUX->mu_lcr = 3;
  REGS_AUX->mu_mcr = 0;

  REGS_AUX->mu_baud_rate = SYSTEM_CLOCK_FREQ / (8 * BAUDRATE) - 1;
  REGS_AUX->mu_control = 3;
}

void uart_pol_putc(char c)
{
  while (!(REGS_AUX->mu_lsr & 0x20)) {
  }

  REGS_AUX->mu_io = c;
}


void myputs(char *str)
{
  while (*str) {
    if (*str == '\n') {
      uart_pol_putc('\r');
    }

    uart_pol_putc(*str);
    str++;
  }
}
